# from rsshub.spiders.zhihu.explore import ctx
# from rsshub.spiders.zhihu.daily import ctx
# from rsshub.spiders.kr36.rss import ctx
from rsshub.spiders.tadoku.books import ctx
# from rsshub.spiders.ruanyifeng.rss import ctx
# from rsshub.spiders.v2x.rss import ctx
# from rsshub.spiders.gscore.rss import ctx
# from rsshub.spiders.bmpi.rss import ctx

from pprint import pprint

def t():
    a = ctx()
    pprint(a)


t()



from flask import current_app, abort
from tests.base import BaseTestCase


class ErrorsTestCase(BaseTestCase):
    def test_400(self):
        @current_app.route('/400')
        def bad_request():
            abort(400)

        response = self.client.get('/400')
        data = response.get_data(as_text=True)
        self.assertIn('400 Bad Request', data)
        self.assertEqual(response.status_code, 400)

    def test_404(self):
        response = self.client.get('/nothing')
        data = response.get_data(as_text=True)
        self.assertIn('404 Not Found', data)
        self.assertEqual(response.status_code, 404)

    def test_500(self):
        @current_app.route('/500')
        def internal_server_error_for_test():
            abort(500)

        response = self.client.get('/500')
        data = response.get_data(as_text=True)
        self.assertIn('服务器出错', data)
        self.assertEqual(response.status_code, 500)



import os
import sys


basedir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


class BaseConfig:
    SITE_NAME = 'RSSHub'
    GITHUB_USERNAME = 'alphardex'
    EMAIL = '2582347430@qq.com'
    SECRET_KEY = os.environ.get('SECRET_KEY') or 'f43hrt53et53'
    DEBUG_TB_INTERCEPT_REDIRECTS = False


class DevelopmentConfig(BaseConfig):
    pass


class TestingConfig(BaseConfig):
    TESTING = True


class ProductionConfig(BaseConfig):
    pass


config = {
    'development': DevelopmentConfig,
    'testing': TestingConfig,
    'production': ProductionConfig
}



"""



"""

import os
import threading
from rsshub import create_app
from argparse import ArgumentParser


parser = ArgumentParser()
parser.add_argument("--ip", type=str, default="127.0.0.1")
parser.add_argument("--port", type=str, default=5005)
parser.add_argument("--https", action="store_true")
parser.add_argument("--debug", action="store_true")
parser.add_argument("--feeder", action="store_true")
args = parser.parse_args()


def main():
    if args.feeder:
        print("enabled feeder, lets feeder!")
        from rsshub.rssfeeder.rss import start_feeds_scheduler

        feeds_thread = threading.Thread(target=start_feeds_scheduler)
        feeds_thread.start()

    app = create_app("production")
    app.run(host=args.ip, port=args.port, debug=args.debug)



import re
from flask import Response
import requests
from parsel import Selector

DEFAULT_HEADERS = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Safari/537.36'}


class XMLResponse(Response):
    def __init__(self, response, **kwargs):
        if 'mimetype' not in kwargs and 'contenttype' not in kwargs:
            if response.startswith('<?xml'):
                kwargs['mimetype'] = 'application/xml'
        return super().__init__(response, **kwargs)


def fetch(url: str, headers: dict=DEFAULT_HEADERS, proxies: dict=None):
    try:
        res = requests.get(url, headers=headers, proxies=proxies)
        res.raise_for_status()
    except Exception as e:
        print(f'[Err] {e}')
    else:
        html = res.text
        tree = Selector(text=html)
        return tree


async def fetch_by_puppeteer(url):
    try:
        from pyppeteer import launch
    except Exception as e:
        print(f'[Err] {e}')
    else:
        browser = await launch(  # 启动浏览器
            {'args': ['--no-sandbox']},
            handleSIGINT=False,
            handleSIGTERM=False,
            handleSIGHUP=False
        )
        page = await browser.newPage()  # 创建新页面
        await page.goto(url)  # 访问网址
        html = await page.content()  # 获取页面内容
        await browser.close()  # 关闭浏览器
        return Selector(text=html)


def filter_content(items):
    content = []
    p1 = re.compile(r'(.*)(to|will|date|schedule) (.*)results', re.IGNORECASE)
    p2 = re.compile(r'(.*)(schedule|schedules|announce|to) (.*)call', re.IGNORECASE)
    p3 = re.compile(r'(.*)release (.*)date', re.IGNORECASE)

    for item in items:
        title = item['title']
        if p1.match(title) or p2.match(title) or p3.match(title):
            content.append(item)
    return content


"""

Automatically upate RSS Hub


- roll_in_rss: function get url, check_rss_url_valid if valid, store it into sqlite, using sqlachemy
- get_rss_content_periodically: 3 h , very 3 hours check new contents from all RSS urls in sql, save new contents into sqlit using sqlachedmy
- gen_new_content_summary: using Ai function get_summary to get summary of each new content, and using ai function get_labels to get labels for content, and labels save to sql
- get_new_articles: serving get the latest new articles, with the lastId from server side, return from lastId -> latestId contents.

"""

from concurrent.futures import ThreadPoolExecutor
import threading
import time
import os
from fastapi import HTTPException
import requests
from sqlalchemy import or_
from .store.dbs import RSSFeed, RSSSource, session
from .utils import (
    check_rss_url_valid,
    extract_img_url_from_html,
    get_rss_info,
    parse_datetime_wisely,
)
from apscheduler.schedulers.background import BackgroundScheduler
import feedparser
from .utils import default_rss_sources
from loguru import logger
from datetime import datetime
from uranuspy.im.client import ChatClient, run_client_forever
from .ai import NewsAgent

uranus_id = os.environ["uranus_id"]
uranus_pwd = os.environ["uranus_pwd"]
assert uranus_id is not None and uranus_pwd is not None, "env need set"
client_uranus = ChatClient(uranus_id, uranus_pwd, broker_address="g.manaai.cn")

news_agent = NewsAgent()


def roll_in_rss(url):
    if check_rss_url_valid(url):
        feed = RSSFeed(url=url)
        session.add(feed)
        session.commit()


def update_rss_content_and_generate_summary():
    sources = session.query(RSSSource).all()
    logger.info(f"{len(sources)} sources to go.")
    # logger.info(sources)
    for i, source in enumerate(sources):
        logger.info(f"resolving: {source.name}, {i}")
        response = requests.get(source.url)
        if response.status_code == 200:
            # Parse the RSS feed content
            feed = feedparser.parse(response.text)
            for entry in feed.entries:
                # Check if the article already exists in the database
                existing_feed = (
                    session.query(RSSFeed)
                    .filter(
                        or_(
                            RSSFeed.url == entry.link,
                            RSSFeed.title.like(f"%{entry.title}%"),
                        )
                    )
                    .first()
                    # .filter(RSSFeed.url == entry.link).first()
                )
                # existing_feed = None
                if not existing_feed:
                    try:
                        new_feed = RSSFeed(
                            url=entry.link,
                            title=entry.title,
                            content=(
                                entry.summary_detail.value
                                if "summary_detail" in entry
                                else entry.description
                            ),
                            summary=(
                                entry.summary
                                if "summary" in entry
                                else entry.description
                            ),
                            source_name=source.name,
                            source_url=source.url,
                            source_avatar_url=source.avatar_url,
                            hero_img_url=extract_img_url_from_html(entry.summary),
                            published=(
                                parse_datetime_wisely(entry.published)
                                if hasattr(entry, "published")
                                else datetime.now()
                            ),
                            created=datetime.now(),
                            author=entry.author if hasattr(entry, "author") else None,
                        )
                    except Exception as e:
                        import traceback

                        logger.error(f"got error: {e}")
                        logger.error(f"got error: {traceback.print_exc()}")
                        continue
                    session.add(new_feed)
                    session.commit()  # Commit here to ensure the new feed has an ID

                    new_feed.ai_summary = news_agent.get_summary(new_feed.content)
                    new_feed.tags = news_agent.get_tags(new_feed.content)
                    # new_feed.ai_summary = ""
                    # new_feed.tags = ""
                    session.commit()

                    logger.info(
                        f"new rss: {new_feed.title} {new_feed.published} {new_feed.author} added."
                    )
                    # broadcast to all uranus users
                    msg_to_send = new_feed.to_json()
                    try:
                        client_uranus.send_rss_news_msg_to_room(
                            msg_to_send, "rss/news_all"
                        )
                    except Exception as e:
                        logger.error(f"sending message error: {e}")
                else:
                    pass
                    # logger.info(f"{existing_feed} already exist.")
        else:
            logger.error(f"request url error: {source.url}")


def save_rss_source(rss_url):
    info = get_rss_info(rss_url)
    if info is not None:
        existing_source = session.query(RSSSource).filter_by(url=rss_url).first()
        # logger.info(existing_source)
        if existing_source is None:
            new_source = RSSSource(
                url=rss_url, name=info.title, avatar_url=info.avatar_url, link=info.link
            )
            session.add(new_source)
            session.commit()
        else:
            existing_source.name = info.title
            existing_source.avatar_url = info.avatar_url
            existing_source.link = info.link
            session.commit()
        return True
    else:
        return False


async def upload_rss_source(request):
    data = await request.json()
    url = data.get("url")
    name = data.get("name")
    avatar_url = data.get("avatar_url")

    if not url:
        raise HTTPException(status_code=400, detail="URL is required")

    # Check if the URL is a valid RSS feed
    try:
        response = requests.get(url)
        response.raise_for_status()
        feed = feedparser.parse(response.text)
        if not feed.entries:
            raise HTTPException(status_code=400, detail="Invalid RSS feed")
    except requests.RequestException:
        raise HTTPException(status_code=400, detail="Invalid URL")

    # Check if the RSS source already exists
    existing_source = session.query(RSSSource).filter_by(url=url).first()
    if existing_source:
        raise HTTPException(status_code=400, detail="RSS source already exists")

    # Create a new RSS source
    new_source = RSSSource(url=url, name=name, avatar_url=avatar_url)
    session.add(new_source)
    session.commit()
    return {"message": "RSS source uploaded successfully", "id": new_source.id}


def save_default_rss_urls():
    for i in default_rss_sources:
        save_rss_source(i)
    logger.info("default rss url initiated.")

    # t1 = run_client_forever(client_uranus)

    # executor = ThreadPoolExecutor(max_workers=1)
    # scheduler = BackgroundScheduler()
    # scheduler.add_job(
    #     lambda: executor.submit(update_rss_content_and_generate_summary),
    #     "interval",
    #     minutes=5,
    # )
    # scheduler.start()
    # t1.join()


def save_rss_url_work():
    print("sleep 60 seconds wait for server up")
    time.sleep(30)
    logger.info("starting scheduler for feeds?")
    save_default_rss_urls()
    logger.info("saved rss urls.")


def start_feeds_scheduler():
    save_thread = threading.Thread(target=save_rss_url_work)
    save_thread.start()

    def run_client(client):
        client.run_forever()

    def run_scheduler():
        # executor = ThreadPoolExecutor(max_workers=1)
        scheduler = BackgroundScheduler()
        scheduler.add_job(
            update_rss_content_and_generate_summary,
            "interval",
            minutes=5,
            max_instances=1,
        )
        scheduler.add_job(
            lambda: logger.info("Scheduler is still running"), "interval", minutes=1
        )
        scheduler.start()
        return scheduler

    save_thread.join()

    scheduler = run_scheduler()
    with ThreadPoolExecutor(max_workers=2) as executor:
        executor.submit(run_client, client_uranus)
        time.sleep(5)
        executor.shutdown(wait=False)
    return scheduler



"""
Simple LLM calling
"""

from typing import Any
import openai


class BaseAgent:

    def __init__(self) -> None:
        self.client = openai.Client(
            base_url="http://g.manaai.cn:8082/v1",
            # api_key="sk-8c945794b98ab61cca14a7ecea0df7bf",
            api_key="sk-6baf45798712e0467d616f4fabff0c51",
        )

    def get_response(self, text) -> Any:
        completion = self.client.chat.completions.create(
            model="qwen2-7b",
            messages=[
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": f"{text}",
                        }
                    ],
                },
            ],
            stream=False,
        )
        return completion.choices[0].message.content


import json
import textwrap
import re


class NewsAgent(BaseAgent):

    def __init__(self) -> None:
        super().__init__()

        self.summary_prompt = textwrap.dedent(
            """你是一个新闻总结专家，请根据这个content总结一下核心内容，要求：
        - 言简意赅，重点信息突出；
        - 合理使用Mark down语法排版；
        - 字数在50字以内
        新闻内容如下：
        """
        )

        self.tags_prompt = textwrap.dedent(
            """你是新闻分类专家，请根据新闻内容给新闻提出2-4个标签，要求：
        - 输出json，格式为:[{"name": "xxx"}, {"name": "xxx"},...] ；
        - 标签第一个为新闻类别，例如时政、体育、娱乐、经济等；
        - 其他标签要突出新闻主体，例如人物、地点、事件；
        - 每个标签字数在4个字以内；
        - 标签数量不要超过4个；
        新闻内容如下：
        """
        )

    def get_summary(self, text):
        a = self.get_response(f"{self.summary_prompt}\n{text}")
        return a

    def get_tags(self, text):
        a = self.get_response(f"{self.tags_prompt}\n{text}")
        pattern = re.compile(r"(?s)(?:```json\n)?(\[.*?\])")
        match = pattern.search(a)
        if match:
            try:
                a = match.group(1)
                b = json.loads(a)
                b = ",".join([list(i.values())[0] for i in b])
            except Exception as e:
                print(f"json parse error: {a} {e}")
                b = ""
        else:
            b = ""
        return b



from flask_bootstrap import Bootstrap
from flask_debugtoolbar import DebugToolbarExtension
from flask_moment import Moment
from flask_caching import Cache


bootstrap = Bootstrap()
debugtoolbar = DebugToolbarExtension()
moment = Moment()

cache = Cache(config={
    "DEBUG": True,          # some Flask specific configs
    "CACHE_TYPE": "simple",  # Flask-Caching related configs
    "CACHE_DEFAULT_TIMEOUT": 3600  # cache half hour
})


from flask import url_for
from tests.base import BaseTestCase


class MainTestCase(BaseTestCase):
    def test_index(self):
        response = self.client.get(url_for('main.index'))
        self.assertEqual(response.status_code, 200)



import unittest
from flask import url_for
from rsshub import create_app


class BaseTestCase(unittest.TestCase):
    def setUp(self):
        app = create_app('testing')
        self.context = app.test_request_context()
        self.context.push()
        self.client = app.test_client()
        self.runner = app.test_cli_runner()

    def tearDown(self):
        self.context.pop()



from tests.base import BaseTestCase


class CLITestCase(BaseTestCase):
    def test_ptshell(self):
        result = self.runner.invoke(args=['ptshell'])
        self.assertNotIn('ptpython not installed! Use the default shell instead.', result.output)



import os
from datetime import datetime
import click
from flask import Flask, render_template
from flask.cli import with_appcontext
from rsshub.config import config
from rsshub.extensions import *
from rsshub.blueprints.main import bp as main_bp
from rsshub.utils import XMLResponse
from rsshub.extensions import cache


def create_app(config_name=None):
    if config_name is None:
        # config_name = os.getenv('FLASK_CONFIG', 'development')
        config_name = os.getenv('FLASK_CONFIG', 'production')

    app = Flask(__name__)
    app.config.from_object(config[config_name])
    app.response_class = XMLResponse
    cache.init_app(app)

    # Add analytics
    # from flask_analytics import Analytics
    from rsshub.google_analytics import ga_account

    # Analytics(app)
    # app.config['ANALYTICS']['GOOGLE_UNIVERSAL_ANALYTICS']['ACCOUNT'] = ga_account
    # app.config['ANALYTICS']['ENABLED'] = True

    register_blueprints(app)
    register_extensions(app)
    register_errors(app)
    register_context_processors(app)
    register_cli(app)

    return app


def register_extensions(app):
    bootstrap.init_app(app)
    debugtoolbar.init_app(app)
    moment.init_app(app)


def register_blueprints(app):
    app.register_blueprint(main_bp)


def register_errors(app):
    @app.errorhandler(400)
    def bad_request(e):
        return render_template('errors/400.html'), 400

    @app.errorhandler(404)
    def page_not_found(e):
        return render_template('errors/404.html'), 404

    @app.errorhandler(500)
    def internal_server_error(e):
        return render_template('errors/500.html'), 500


def register_context_processors(app):
    @app.context_processor
    def inject_date_now():
        now = datetime.utcnow()
        return {'now': now}


def register_cli(app):
    @app.cli.command()
    @with_appcontext
    def ptshell():
        """Use ptpython as shell."""
        try:
            from ptpython.repl import embed
            if not app.config['TESTING']:
                embed(app.make_shell_context())
        except ImportError:
            click.echo('ptpython not installed! Use the default shell instead.')



"""

Utils for RSS
"""

from datetime import datetime
from typing import Optional
from bs4 import BeautifulSoup
import feedparser
from pydantic import BaseModel
import requests
import dateutil.parser


class RSSSource(BaseModel):
    title: str
    link: str
    avatar_url: Optional[str] = None


def parse_datetime_wisely(date_string):
    """
    尝试解析多种格式的日期时间字符串。
    如果所有尝试都失败，则返回当前时间。
    """
    if not date_string:
        return datetime.now()

    # 首先尝试使用 dateutil 解析
    try:
        return dateutil.parser.parse(date_string)
    except (ValueError, TypeError):
        pass

    # 定义一系列常见的日期时间格式
    formats = [
        "%a, %d %b %Y %H:%M:%S %Z",  # RFC 822
        "%Y-%m-%dT%H:%M:%S%z",  # ISO 8601
        "%Y-%m-%d %H:%M:%S.%f",  # 带微秒的标准格式
        "%Y-%m-%d %H:%M:%S",  # 标准格式
        "%Y-%m-%d",  # 仅日期
        "%d %b %Y %H:%M:%S %z",  # 另一种常见格式
    ]

    for fmt in formats:
        try:
            return datetime.strptime(date_string, fmt)
        except ValueError:
            continue

    # 如果所有尝试都失败，记录错误并返回当前时间
    print(f"Unable to parse date string: {date_string}")
    return datetime.now()


def extract_img_url_from_html(s):
    soup = BeautifulSoup(s, "html.parser")
    img_tags = soup.find_all("img")
    img_urls = [img["src"] for img in img_tags if "src" in img.attrs]
    if len(img_urls) > 0:
        return img_urls[0]
    else:
        return ""


def get_avatar_url(homepage_url):
    try:
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3"
        }
        response = requests.get(homepage_url, headers=headers)
        response.raise_for_status()
        soup = BeautifulSoup(response.text, "html.parser")

        # Look for the favicon link in the head
        favicon_link = soup.find("link", rel="icon") or soup.find(
            "link", rel="shortcut icon"
        )

        if favicon_link and "href" in favicon_link.attrs:
            favicon_url = favicon_link["href"]
            if favicon_url.startswith("//"):
                favicon_url = "http:" + favicon_url
            else:
                if not favicon_url.startswith("http"):
                    # Handle relative URLs
                    favicon_url = requests.compat.urljoin(homepage_url, favicon_url)
            return favicon_url
        else:
            return None
    except Exception as e:
        print(f"获取头像 URL 失败: {e}")
        return None


def get_rss_info(url):
    try:
        feed = feedparser.parse(url)
        if feed.entries:
            # print(f"==> RSS 源 {url} 有效，包含 {len(feed.entries)} 条目")
            # print(f"==> RSS info: {feed.feed}")
            # print(feed.entries[0])
            # save avatar url?
            link = feed.feed.link
            # avatar_url = get_avatar_url(homepage_url)
            avatar_url = get_avatar_url(link)
            print(f"avatar_url: {avatar_url}, url: {url} {link}")
            return RSSSource(
                title=feed.feed.title, link=feed.feed.link, avatar_url=avatar_url
            )
        else:
            print(f"RSS 源 {url} 接通但内容无效")
            return None
    except Exception as e:
        print(f"RSS 源 {url} 接通失败: {e}")
        return None


def check_rss_url_valid(url):
    try:
        feed = feedparser.parse(url)
        if feed.entries:
            print(f"==> RSS 源 {url} 有效，包含 {len(feed.entries)} 条目")
            print(f"==> RSS info: {feed.feed}")
            print(feed.entries[0])
            return True
        else:
            print(f"RSS 源 {url} 接通但内容无效")
            return False
    except Exception as e:
        print(f"RSS 源 {url} 接通失败: {e}")
        return False


"""
https://status.uri.wang/status/rsshub
"""

# rss_url = "i.scnu.edu.cn/sub"
# rss_url = "https://rss.injahow.cn"
rss_url = "http://127.0.0.1:5005"  # default port

default_rss_sources = [
    # f"{rss_url}/apnews/rss/business",
    # f"{rss_url}/pixiv/search/Nezuko/popular",
    # f"{rss_url}/caixinglobal/latest",
    # f"{rss_url}/36kr/hot-list",
    # f"{rss_url}/19lou/jiaxing",
    # f"{rss_url}/aliresearch/information",
    # f"{rss_url}/dedao/list/年度日更",
    # f"{rss_url}/guancha/headline",
    # f"{rss_url}/guancha/",
    # f"{rss_url}/geekpark/breakingnews/",
    # f"{rss_url}/zhihu/hotlist/",
    # f"{rss_url}/guokr/scientific/",
    # f"{rss_url}/cctv/world/",
    # f"{rss_url}/xueqiu/user/6410914956",
    # f"{rss_url}/xueqiu/user/2852344450",
    # f"{rss_url}/kkj/news",
    # f"{rss_url}/huxiu/briefcolumn/1",
    # f"{rss_url}/xinhuanet/yaodianjujiao",
    f"{rss_url}/cls/telegraph",
    f"{rss_url}/tadoku/books/l0",
    f"{rss_url}/infoq/recommend",
    f"{rss_url}/bbwc/realtime/2",
    f"{rss_url}/zhihu/daily",
    f"{rss_url}/gscore/rss",
    f"{rss_url}/ruanyifeng/rss",
    # f"{rss_url}/36kr/rss",
    f"{rss_url}/taimeiti/rss",
    f"{rss_url}/huxiu/rss",
    f"{rss_url}/geekpark/rss",
]



import os
from setuptools import setup, find_packages


_PATH_ROOT = os.path.dirname(__file__)

with open(os.path.join(_PATH_ROOT, "README.md"), encoding="utf-8") as fo:
    readme = fo.read()

setup(
    name="rssreborn",
    version="0.1.1",
    description="Make RSS great again!",
    author="Open Jarvis AI",
    url="https://github.com/lucasjinreal/RSSReborn",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "rssreborn=rsshub.rssreborn:main",
        ],
    },
    install_requires=[
        "openai",
        "Flask>=2.0.2",
        "Flask-Analytics>=0.6.0",
        "Flask-Caching>=2.0.2",
        "Flask-DebugToolbar>=0.11.0",
        "Flask-Moment>=1.0.2",
        "Flask-Script>=2.0.6",
        "gunicorn>=20.1.0",
        "icecream>=2.1.1",
        "arrow",
        "pyjsparser"
    ],
    long_description=readme,
    long_description_content_type="text/markdown",
)



# import json
import json
from sqlalchemy import (
    DateTime,
    ForeignKey,
    create_engine,
    Column,
    Integer,
    String,
    Text,
)
import os
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
import feedparser
import requests
from datetime import datetime
from sqlalchemy.orm import relationship, sessionmaker

os.makedirs(os.path.expanduser("~/data"), exist_ok=True)
os.makedirs("./data", exist_ok=True)
Base = declarative_base()


class RSSSource(Base):
    __tablename__ = "rss_source"
    id = Column(Integer, primary_key=True)
    url = Column(String, unique=True)
    link = Column(String, unique=True)
    name = Column(Text)
    avatar_url = Column(Text)


class RSSFeed(Base):
    __tablename__ = "rss_feeds"
    id = Column(Integer, primary_key=True)
    url = Column(String, unique=True)
    content = Column(Text)
    title = Column(String)
    hero_img_url = Column(String)
    published = Column(DateTime)
    created = Column(DateTime)
    author = Column(String)
    source_name = Column(String)
    source_url = Column(String)
    source_avatar_url = Column(String)
    summary = Column(Text)
    ai_summary = Column(Text)  # Field for AI-generated summary
    tags = Column(Text)  # Field for AI-generated labels
    num_looked = Column(
        Integer, default=0
    )  # Number of times the feed has been looked at
    comments = relationship("Comment", back_populates="feed")

    def to_dict(self):
        return {
            "id": self.id,
            "url": self.url,
            "content": self.content,
            "title": self.title,
            "published": self.published.isoformat() if self.published else None,
            "created": self.created.isoformat() if self.created else None,
            "author": self.author,
            "source_name": self.source_name,
            "source_url": self.source_url,
            "source_avatar_url": self.source_avatar_url,
            "summary": self.summary,
            "ai_summary": self.ai_summary,
            "tags": self.tags,
            "num_looked": self.num_looked,
            "comments": self.comments,
            "hero_img_url": self.hero_img_url,
        }

    def to_json(self):
        return json.dumps(self.to_dict(), indent=2, ensure_ascii=False)


class Comment(Base):
    __tablename__ = "comments"
    id = Column(Integer, primary_key=True)
    feed_id = Column(Integer, ForeignKey("rss_feeds.id"))
    user_name = Column(String)
    user_avatar_url = Column(String)
    comment_time = Column(DateTime, default=datetime.utcnow)
    comment_content = Column(Text)
    feed = relationship("RSSFeed", back_populates="comments")


# sql_p = os.path.join(os.path.expanduser("~/data"), "rss_feeds.db")
sql_p = "data/rss_feeds.db"
engine = create_engine(f"sqlite:///{sql_p}")
print(f"sqlite path: {sql_p}")
Base.metadata.create_all(engine)
Session = sessionmaker(bind=engine)
session = Session()



from flask import Blueprint, jsonify, render_template, request
from rsshub.extensions import cache

from rsshub.rssfeeder.rss import save_rss_source
from rsshub.rssfeeder.store.dbs import RSSFeed, RSSSource, session
from sqlalchemy import text


bp = Blueprint("main", __name__)


@bp.route("/add_rss", methods=["POST"])
def add_rss():
    save_rss_source()
    data = request.get_json()
    rss_url = data.get("rss_url")

    if not rss_url:
        return jsonify({"error": "RSS URL is required"}), 400

    res = save_rss_source(rss_url=rss_url)
    if res:
        return jsonify({"status": "ok", "message": "rss添加成功"}), 201
    else:
        return (
            jsonify({"status": "error", "message": "rss解析失败，请确保rss源是否有效"}),
            400,
        )


@bp.route("/news", methods=["GET"])
def get_paginated_news():
    page = request.args.get("page", default=1, type=int)
    per_page = request.args.get("per_page", default=30, type=int)
    offset = (page - 1) * per_page
    paginated_feeds = (
        session.query(RSSFeed)
        .order_by(RSSFeed.created.desc())
        .offset(offset)
        .limit(per_page)
        .all()
    )

    result = {
        "message": "you rss news",
        "status": "ok",
        "num": len(paginated_feeds),
        "data": [r.to_json() for r in paginated_feeds],
    }
    return jsonify(result)


@bp.route("/word/<string:category>")
@bp.route("/")
@cache.cached(timeout=3600)
def word(category=""):
    from rsshub.spiders.word.word import ctx

    return render_template("main/word.html", **ctx(category))


@bp.route("/index")
def index():
    return render_template("main/index.html")


@bp.route("/feeds")
def feeds():
    return render_template("main/feeds.html")


@bp.app_template_global()
def filter_content(ctx):
    include_title = request.args.get("include_title")
    include_description = request.args.get("include_description")
    exclude_title = request.args.get("exclude_title")
    exclude_description = request.args.get("exclude_description")
    limit = request.args.get("limit", type=int)
    items = ctx["items"].copy()

    if include_title:
        include_keywords = (
            include_title.split("|") if "|" in include_title else [include_title]
        )
        items = [
            item
            for item in items
            if any(keyword in item["title"] for keyword in include_keywords)
        ]

    if include_description:
        include_keywords = (
            include_description.split("|")
            if "|" in include_description
            else [include_description]
        )
        items = [
            item
            for item in items
            if any(keyword in item["description"] for keyword in include_keywords)
        ]

    if exclude_title:
        exclude_keywords = (
            exclude_title.split("|") if "|" in exclude_title else [exclude_title]
        )
        items = [
            item
            for item in items
            if all(keyword not in item["title"] for keyword in exclude_keywords)
        ]

    if exclude_description:
        exclude_keywords = (
            exclude_description.split("|")
            if "|" in exclude_description
            else [exclude_description]
        )
        items = [
            item
            for item in items
            if all(keyword not in item["description"] for keyword in exclude_keywords)
        ]

    if limit:
        items = items[:limit]

    ctx = ctx.copy()
    ctx["items"] = items
    return ctx


# ---------- feed路由从这里开始 -----------#
@bp.route("/gscore/rss")
def gscore_rss():
    from rsshub.spiders.gscore.rss import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/ruanyifeng/rss")
def ruanyifeng_rss():
    from rsshub.spiders.ruanyifeng.rss import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/36kr/rss")
def kr36_rss():
    from rsshub.spiders.kr36.rss import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/taimeiti/rss")
def taimeiti_rss():
    from rsshub.spiders.taimeiti.rss import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/huxiu/rss")
def huxiu_rss():
    from rsshub.spiders.huxiu.rss import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/geekpark/rss")
def geekpark_rss():
    from rsshub.spiders.geekpark.rss import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/zhihu/explore")
def zhihu_explore():
    from rsshub.spiders.zhihu.explore import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/zhihu/daily")
def zhihu_daily():
    from rsshub.spiders.zhihu.daily import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/zhihu/question/<string:qid>")
def zhihu_question(qid):
    from rsshub.spiders.zhihu.article import ctx_question

    return render_template("main/atom.xml", **filter_content(ctx_question(qid)))


@bp.route("/cninfo/announcement/<string:stock_id>/<string:category>")
@bp.route("/cninfo/announcement")
def cninfo_announcement(stock_id="", category=""):
    from rsshub.spiders.cninfo.announcement import ctx

    return render_template("main/atom.xml", **filter_content(ctx(stock_id, category)))


@bp.route("/chuansongme/articles/<string:category>")
@bp.route("/chuansongme/articles")
def chuansongme_articles(category=""):
    from rsshub.spiders.chuansongme.articles import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/ctolib/topics/<string:category>")
@bp.route("/ctolib/topics")
def ctolib_topics(category=""):
    from rsshub.spiders.ctolib.topics import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/bbwc/realtime/<string:category>")
def bbwc_realtime(category=""):
    from rsshub.spiders.bbwc.realtime import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/infoq/recommend")
def infoq_recommend():
    from rsshub.spiders.infoq.recommend import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/infoq/topic/<int:category>")
def infoq_topic(category=""):
    from rsshub.spiders.infoq.topic import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/infoq/profile/<string:category>")
def infoq_profile(category=""):
    from rsshub.spiders.infoq.profile import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/infoq/search/<string:category>/<int:type>")
def infoq_search(category="", type=""):
    from rsshub.spiders.infoq.search import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category, type)))


@bp.route("/dxzg/notice")
def dxzg_notice():
    from rsshub.spiders.dxzg.notice import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/earningsdate/prnewswire")
def earningsdate_prnewswire():
    from rsshub.spiders.earningsdate.prnewswire import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/earningsdate/globenewswire")
def earningsdate_globenewswire():
    from rsshub.spiders.earningsdate.globenewswire import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/earningsdate/businesswire")
def earningsdate_businesswire():
    from rsshub.spiders.earningsdate.businesswire import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/jiemian/newsflash/<string:category>")
def jiemian_newsflash(category=""):
    from rsshub.spiders.jiemian.newsflash import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/csrc/audit/<string:category>")
def csrc_audit(category=""):
    from rsshub.spiders.csrc.audit import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/caixin/scroll/<string:category>")
def caixin_scroll(category=""):
    from rsshub.spiders.caixin.scroll import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/eastmoney/report/<string:type>/<string:category>")
def eastmoney_report(category="", type=""):
    from rsshub.spiders.eastmoney.report import ctx

    return render_template("main/atom.xml", **filter_content(ctx(type, category)))


@bp.route("/xuangubao/<string:type>/<string:category>")
def xuangubao_xuangubao(type="", category=""):
    from rsshub.spiders.xuangubao.xuangubao import ctx

    return render_template("main/atom.xml", **filter_content(ctx(type, category)))


@bp.route("/cls/subject/<string:category>")
def cls_subject(category=""):
    from rsshub.spiders.cls.subject import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/cls/telegraph/")
def cls_telegraph():
    from rsshub.spiders.cls.telegraph import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/chaindd/column/<string:category>")
def chaindd_column(category=""):
    from rsshub.spiders.chaindd.column import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/techcrunch/tag/<string:category>")
def techcrunch_tag(category=""):
    from rsshub.spiders.techcrunch.tag import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/weiyangx/home")
def weiyangx_home():
    from rsshub.spiders.weiyangx.home import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/weiyangx/express/")
def weiyangx_express():
    from rsshub.spiders.weiyangx.express import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/weiyangx/tag/<string:category>")
def weiyangx_tag(category=""):
    from rsshub.spiders.weiyangx.tag import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/jintiankansha/column/<string:category>")
def jintiankansha_column(category=""):
    from rsshub.spiders.jintiankansha.column import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/interotc/cpgg/<string:category>")
def interotc_cpgg(category=""):
    from rsshub.spiders.interotc.cpgg import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/benzinga/ratings/<string:category>")
def benzinga_ratings(category=""):
    from rsshub.spiders.benzinga.ratings import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/chouti/section/<string:category>")
def chouti_section(category=""):
    from rsshub.spiders.chouti.section import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/chouti/search/<string:category>")
def chouti_search(category=""):
    from rsshub.spiders.chouti.search import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/chouti/user/<string:category>")
def chouti_user(category=""):
    from rsshub.spiders.chouti.user import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/zaobao/realtime/<string:category>")
def zaobao_realtime(category=""):
    from rsshub.spiders.zaobao.realtime import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/mp/tag/<string:mp>/<string:tag>")
def mp_tag(mp="", tag=""):
    from rsshub.spiders.mp.tag import ctx

    return render_template("main/atom.xml", **filter_content(ctx(mp, tag)))


@bp.route("/mp/rtag/<string:c1>/<string:tag>")
def mp_rtag(c1="", tag=""):
    from rsshub.spiders.mp.rtag import ctx

    return render_template("main/atom.xml", **filter_content(ctx(c1, tag)))


@bp.route("/producthunt/search/<string:keyword>/<string:period>")
def producthunt_search(keyword="", period=""):
    from rsshub.spiders.producthunt.search import ctx

    return render_template("main/atom.xml", **filter_content(ctx(keyword, period)))


@bp.route("/pgyer/<string:category>")
def pgyer_app(category=""):
    from rsshub.spiders.pgyer.app import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/economist/worldbrief")
def economist_wordlbrief(category=""):
    from rsshub.spiders.economist.worldbrief import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/baidu/suggest/<string:category>")
def baidu_suggest(category=""):
    from rsshub.spiders.baidu.suggest import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/mp/gh/<string:gh>")
def mp_gh(gh=""):
    from rsshub.spiders.mp.gh import ctx

    return render_template("main/atom.xml", **filter_content(ctx(gh)))


@bp.route("/mp/youwuqiong/<string:author>")
def mp_youwuqiong(author=""):
    from rsshub.spiders.mp.youwuqiong import ctx

    return render_template("main/atom.xml", **filter_content(ctx(author)))


@bp.route("/xinhuanet/zuixinbobao")
def xinhuanet_zuixinbobao():
    from rsshub.spiders.xinhuanet.zuixinbobao import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/xinhuanet/shizhenglianbo")
def xinhuanet_shizhenglianbo():
    from rsshub.spiders.xinhuanet.shizhenglianbo import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/xinhuanet/yaodianjujiao")
def xinhuanet_yaodianjujiao():
    from rsshub.spiders.xinhuanet.yaodianjujiao import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/xinhuanet/world")
def xinhuanet_world():
    from rsshub.spiders.xinhuanet.world import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/yfchuhai/express/")
def yfchuhai_express():
    from rsshub.spiders.yfchuhai.express import ctx

    return render_template("main/atom.xml", **filter_content(ctx()))


@bp.route("/bjnews/<string:category>")
def bjnews_channel(category=""):
    from rsshub.spiders.bjnews.channel import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/appstore/top/<string:cc>/<string:genreid>")
def appstore_top(cc="", genreid=""):
    from rsshub.spiders.appstore.top import ctx

    return render_template("main/atom.xml", **filter_content(ctx(cc, genreid)))


@bp.route("/netease/comment/<string:category>")
def netease_comment(category=""):
    from rsshub.spiders.netease.comment import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/aisixiang/search/<string:category>/<string:keywords>")
def aisixiang_search(category="", keywords=""):
    from rsshub.spiders.aisixiang.search import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category, keywords)))


@bp.route("/hnzcy/bidding/<string:type>")
@cache.cached(timeout=3600)
def hnzcy_bidding(type=""):
    from rsshub.spiders.hnzcy.bidding import ctx

    return render_template("main/atom.xml", **filter_content(ctx(type)))


@bp.route("/sysu/ifcen")
@cache.cached(timeout=3600)
def sysu_ifcen(category="", keywords=""):
    from rsshub.spiders.sysu.ifcen import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/nhk/newseasy")
@cache.cached(timeout=3600)
def nhk_newseasy(category="", keywords=""):
    from rsshub.spiders.nhk.newseasy import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/nhk/topic/<string:category>")
@cache.cached(timeout=3600)
def nhk_topic(category="", keywords=""):
    from rsshub.spiders.nhk.topic import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/tadoku/books/<string:category>")
@cache.cached(timeout=3600)
def tadoku_books(category=""):
    from rsshub.spiders.tadoku.books import ctx

    return render_template("main/atom.xml", **filter_content(ctx(category)))


@bp.route("/filter/")
def rss_filter():
    from rsshub.spiders.rssfilter.filter import ctx

    feed_url = request.args.get("feed")
    return render_template("main/atom.xml", **filter_content(ctx(feed_url)))
